/* Copyright (c) 2023 Skyward Experimental Rocketry
 * Author: Nicolò Caruso
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <sensors/SensorData.h>

namespace Boardcore
{

struct MY_LIS3DSHData : public AccelerometerData
{
    /*! Struct default inistialization / constructor */
    MY_LIS3DSHData() : AccelerometerData(0, 0, 0, 0){};

    /*! Constructor with Timestamp, accelerations on X,Y,Z axis */
    MY_LIS3DSHData(uint64_t ts, float aX, float aY, float aZ)
        : AccelerometerData(ts, aX, aY, aZ){};

    /*! Accelerometer data as input constructor */
    explicit MY_LIS3DSHData(AccelerometerData acc) : AccelerometerData(acc){};

    /*! Struct header */
    static std::string header()
    {
        return "timestamp, accelerationX, accelerationY, accelerationZ\n";
    }

    /*! Constant method to print to given stream the datastr. */
    void print(std::ostream& os) const
    {
        os << accelerationTimestamp << ", " << accelerationX << ", "
           << accelerationY << ", " << accelerationZ << "\n";
    }
};
}  // namespace Boardcore