/* Copyright (c) 2023 Skyward Experimental Rocketry
 * Author: Nicolò Caruso
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#pragma once
#include "BufferedFlashBackend.h"

namespace Boardcore
{

void BufferedFlashBackend::write(std::vector<uint8_t> &vector)
{
    std::lock_guard<std::mutex> lockBuffers(mutexBuffers);
    std::lock_guard<std::mutex> lockCurrent(currentBuffer->mutex);

    // Copy the vector to be then written and tell there is a needed write
    currentBuffer->vector = vector;
    currentBuffer->write  = true;
}

void BufferedFlashBackend::swap()
{
    std::lock_guard<std::mutex> lockBuffers(mutexBuffers);

    WriteBuffer *tmp;
    tmp           = busyBuffer;
    busyBuffer    = currentBuffer;
    currentBuffer = tmp;
}

void BufferedFlashBackend::run()
{
    while (true)
    {
        // Writes the busy buffer when it needs to be written
        {
            std::unique_lock<std::mutex> lockBusy(busyBuffer->mutex);

            // Waits until the busy buffer not needs to be written
            while (!busyBuffer->write)
                writeCondition.wait(lockBusy);

            // TODO: write to backend the busy buffer
            busyBuffer->write = false;
        }

        // Swaps the buffer to update and the one to be written
        swap();
    }
}

bool BufferedFlashBackend::load(std::vector<uint8_t> &vector)
{
    // TODO: will call the backend for load the configuration to the vector
    return false;
}

void BufferedFlashBackend::clear()
{
    std::lock_guard<std::mutex> lockBuffers(mutexBuffers);
    std::lock_guard<std::mutex> lockBusy(busyBuffer->mutex);
    std::lock_guard<std::mutex> lockCurrent(currentBuffer->mutex);

    busyBuffer->vector.clear();
    currentBuffer->vector.clear();

    busyBuffer->write    = false;
    currentBuffer->write = false;
}

}  // namespace Boardcore