/* Copyright (c) 2023 Skyward Experimental Rocketry
 * Author: Nicolò Caruso
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <utils/Debug.h>

namespace Boardcore
{

/**
 * Macro for the creation of the used datastructures, they will be created by
 * using CONF(NAME_ENTRY, type) The data structure will contain its
 * configuration enum entry, the datatype and the value of the configuration.
 */
#define CONF(name, dataType)                       \
    struct name                                    \
    {                                              \
        using type = dataType;                     \
        dataType value;                            \
        ConfigurationEnum enumIndex = name##_CONF; \
    };

/**
 * These are the possible configuration entries structures, with their value
 * type */
CONF(RIG_TIMING1, float)
CONF(RIG_TIMING2, int)

/**
 * Configuration enum, includes all the possible configurations accepted by the
 * front-end for saving. Each own has its data structure, created by using the
 * macro. By adding one configuration also an equivalent enum named with pedix
 * _CONF must be added*/
enum ConfigurationEnum
{
    RIG_TIMING1_CONF,
    RIG_TIMING2_CONF
};

/**
 * This
 */
class RegistryFrontEndInterface
{
public:
    virtual ~RegistryFrontEndInterface(){};

    virtual
};

}  // namespace Boardcore